﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Controls.Menus;
using Infragistics.Windows.DataPresenter;

namespace DiffernetXamContextMenuBasesOnClickedItem
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent(); 
        }
    }

    public class MainViewModel : PropertyChangedNotifier
    {

        private ObservableCollection<ParentViewModel> gridData;
        private ObservableCollection<MenuViewModel> rootMenu;
        private ObservableCollection<MenuViewModel> childMenu;
        private ObservableCollection<MenuViewModel> parentMenu;

        public MainViewModel()
        {
            GridData = new ObservableCollection<ParentViewModel>();
            for (int i = 0; i < 10; i++)
            {
                ParentViewModel pvm = new ParentViewModel
                {
                    ID = i,
                    Name = "Parent " + i,
                    Email = "parent" + i + "@infragistics.com",
                    Children = new ObservableCollection<ChildViewModel>()
                };

                for (int j = 0; j < 5; j++)
                {
                    pvm.Children.Add(new ChildViewModel
                    {
                        ChildID = j,
                        Name = "Child " + j,
                        Age = 10 + i + j
                    });
                }

                GridData.Add(pvm);

                RootMenu = new ObservableCollection<MenuViewModel>
                {
                    new MenuViewModel
                    {
                        Header = "Root",
                        Command = new RelayCommand<object>(
                            new Action<object>(
                                (obj)=>
                                    {
                                        MessageBox.Show("Empty Area Menu");
                                    }))
                    }
                };

                ParentMenu = new ObservableCollection<MenuViewModel>
                {
                    new MenuViewModel
                    {
                        Header = "Parent",
                        Command = new RelayCommand<ParentViewModel>(
                            new Action<ParentViewModel>(
                                (parent)=>
                                    {
                                        MessageBox.Show("Clicked: " + parent.Name);
                                    }))
                    }
                };

                ChildMenu = new ObservableCollection<MenuViewModel>
                {
                    new MenuViewModel
                    {
                        Header = "Child",
                        Command = new RelayCommand<ChildViewModel>(
                            new Action<ChildViewModel>(
                                (child)=>
                                    {
                                        MessageBox.Show("Clicked: " + child.Name);
                                    }))
                    }
                };
            }
        }

        public ObservableCollection<ParentViewModel> GridData
        {
            get { return gridData; }
            set { gridData = value; OnPropertyChagned("GridData"); }
        }


        public ObservableCollection<MenuViewModel> RootMenu
        {
            get { return rootMenu; }
            set { rootMenu = value; OnPropertyChagned("RootMenu"); }
        }


        public ObservableCollection<MenuViewModel> ParentMenu
        {
            get { return parentMenu; }
            set { parentMenu = value; OnPropertyChagned("ParentMenu"); }
        }


        public ObservableCollection<MenuViewModel> ChildMenu
        {
            get { return childMenu; }
            set { childMenu = value; OnPropertyChagned("ChildMenu"); }
        }

    }

    public class MenuViewModel : PropertyChangedNotifier
    {
        private string header;

        public string Header
        {
            get { return header; }
            set { header = value; OnPropertyChagned("Header"); }
        }

        private ICommand command;

        public ICommand Command
        {
            get { return command; }
            set { command = value; OnPropertyChagned("Command"); }
        }

    }

    public class ParentViewModel : PropertyChangedNotifier
    {
        private int id;

        public int ID
        {
            get { return id; }
            set { id = value; OnPropertyChagned("ID"); }
        }

        private string name;

        public string Name
        {
            get { return name; }
            set { name = value; OnPropertyChagned("Name"); }
        }

        private string email;

        public string Email
        {
            get { return email; }
            set { email = value; OnPropertyChagned("Email"); }
        }

        private ObservableCollection<ChildViewModel> children;

        public ObservableCollection<ChildViewModel> Children
        {
            get { return children; }
            set { children = value; OnPropertyChagned("Children"); }
        }


    }

    public class ChildViewModel : PropertyChangedNotifier
    {
        private int childID;

        public int ChildID
        {
            get { return childID; }
            set { childID = value; OnPropertyChagned("ChildID"); }
        }

        private string name;

        public string Name
        {
            get { return name; }
            set { name = value; OnPropertyChagned("Name"); }
        }

        private int age;

        public int Age
        {
            get { return age; }
            set { age = value; OnPropertyChagned("Age"); }
        }

    }

    public class PropertyChangedNotifier : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChagned(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }

    public class RelayCommand<T> : ICommand
    {

        #region Declarations

        readonly Predicate<T> _canExecute;
        readonly Action<T> _execute;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class and the command can always be executed.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        public RelayCommand(Action<T> execute)
            : this(execute, null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        /// <param name="canExecute">The execution status logic.</param>
        public RelayCommand(Action<T> execute, Predicate<T> canExecute)
        {

            if (execute == null)
                throw new ArgumentNullException("execute");
            _execute = execute;
            _canExecute = canExecute;
        }

        #endregion

        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add
            {

                if (_canExecute != null)
                    CommandManager.RequerySuggested += value;
            }
            remove
            {

                if (_canExecute != null)
                    CommandManager.RequerySuggested -= value;
            }
        }
         
        public virtual Boolean CanExecute(Object parameter)
        {
            return _canExecute == null ? true : _canExecute((T)parameter);
        }

        public virtual void Execute(Object parameter)
        {
            _execute((T)parameter);
        }

        #endregion
    }

}
